# Instruction to run VFC package from mileage prediction model to MDCEV model
#
# 1. Select from line 10 to 37 and run to load libraries and read input data
# 2. Select from line 40 to 59 and then run for a mileage prediction model
# 3. Select from line 62 to 91 and then run for a MNL model for the number of alternatives
# 4. Select from line 94 to 237 and then run MDCEV model
# 5. Select from line 239 to 305 and then run for Heuristic mileage allocation and a MNL model for the number of vehicle-body type
# 6. Select from line 309 to the end and then run for all count models (i.e. Car, VAN, SUV, and Pick-up Truck)

library("utils")       # to use csv read function
library("foreign")     # to use spss read function
library("pastecs")     # to use descriptive statistics function
library("mlogit")
library("graphics")
library("VGAM")
library("ZeligChoice") # This package is for actual mlogit function
library("aod")
library("plotrix")
library("MASS")        # For ordered Probit
library("ordinal")     # Also for ordered probit
library("lmtest")      # for likelihood ratio test
library("miscTools")
library("maxLik")


# Set project folder and path of Halton file and input file
current_folder <<- "C:/workspace/VFC_Package_Beta/Model_Application/";
Halton_File <<- "C:/workspace/VFC_Package_Beta/Halton_R/haltbrat_vfc101.csv";
input_file <- "C:/workspace/VFC_Package_Beta/Model_Application/az_hhld_vfc_cleaned_final.csv";

# Set path of output files
mileage_output <- "C:/workspace/VFC_Package_Beta/Model_Application/VFC_Output.csv";
num_alternative_output <- "C:/workspace/VFC_Package_Beta/Model_Application/VFC_Output.csv";
mdcev_output <- "C:/workspace/VFC_Package_Beta/Model_Application/VFC_Output.csv";

# Read input data
data1 <<- read.table(input_file, header=T, sep=",");


# 1 - Mileage Prediction Model

coefs_name_mile <- c('const','DRVRCNT','NUMADLT','RUR','INCOME1','INCOME5','NUMCHILD','WORK0','WORK2','HSIZE4','SF_Own','Retired','Prop_HH_INC5', 'Prop_SFHH', 'AUTO10_Q1');
coefs_mile <- c(13.03, 2.01, 0.374, 0.867, -1.05, 1.325, 0.517, -1.513, 0.829, -0.669, 0.570, -0.878, 1.362, 0.687, -0.293);

print("Running mileage prediction model .......");
source(paste(c(current_folder,'1_Mileage_prediction.r'),collapse=''));

tt1 <- mileage_model(data1, coefs_name_mile, coefs_mile);

#tt1$ID <- data1$id;
#tt1$const <- NULL;
tt1$omiles <- NULL;
tt1$obs_mile <- NULL;
tt1$pre_mile <- NULL;

data1 <<- cbind(data1,tt1);

write.table(data1, file = mileage_output, row.names=FALSE, sep=",");
rm(tt1);


# 2 - MNL model for the number of alternative

print("Running MNL model for the number of alternatives .......");
source(paste(c(current_folder,'2_Number_of_alternatives.r'),collapse=''));

coefs_name_0veh <- c('const', 'INCOME1', 'INCOME2', 'OWN', 'HSIZE1', 'WORK0', 'POPDEN');
coefs_0veh <- c(1.73, 2.37, 0.82, -1.65, 2.12, 1.21, 0.00011);
coefs_name_1veh <- c('const', 'INCOME1', 'INCOME2', 'HSIZE1', 'Prop_MFHH', 'WORK2');
coefs_1veh <- c(4.27, 1.43, 0.96, 2.22, 0.4, -0.86);
coefs_name_2veh <- c('const', 'lif_cyc_1', 'INCOME3', 'WORK2', 'INC1');
coefs_2veh <- c(5.31, 0.42, -0.21, -0.34, -0.00034);
coefs_name_3veh <- c('const', 'OWN', 'NUMADLT', 'WORK3', 'POPDEN', 'child_pres', 'lif_cyc_2');
coefs_3veh <- c(1.69, 1.0, 0.48, 1.02, -0.000088, 0.38, 0.5);
coefs_name_4veh <- c('const', 'INCOME5', 'lif_cyc_2', 'HSIZE4', 'WORK2');
coefs_4veh <- c(1.18, 0.72, 1.63, 1.51, -0.7);
coefs_name_5veh <- c('const');
coefs_5veh <- c(0.0);

coef_names_m2 <- list(coefs_name_0veh, coefs_name_1veh, coefs_name_2veh , coefs_name_3veh, coefs_name_4veh, coefs_name_5veh);
coef_values_m2 <- list(coefs_0veh, coefs_1veh, coefs_2veh, coefs_3veh, coefs_4veh, coefs_5veh);

tt1 <- NumAlternative_model(data1, coef_names_m2, coef_values_m2);

#tt1$ID <- data1$id;
tt1$num_obs <- NULL;

data1 <<- cbind(data1,tt1);

write.table(data1, file = num_alternative_output, row.names=FALSE, sep=",");
rm(tt1);


# 3 - MDCEV forcasting model

print("Running MDCEV forecasting code.......");
source(paste(c(current_folder,'3_Vehicle_fleet_mix.r'),collapse=''));

#### Control Variables for MDCEV ####

controls <- list();
controls[[1]] <- 5;  # UNO - One (Column Index)
controls[[2]] <- 6;  # SERO - Zero (Column Index)
controls[[3]] <- 7;  # Case ID (Column Index)
controls[[4]] <- 25; # Number of Replications
controls[[5]] <- 1;  # Number of Outside Goods
controls[[6]] <- 4;  # Configuration (1,4,5,6, or 7)


#### Baseline utility equation
alt_names <- c("Car 0-5","Car 6-11","Car 12 or older","Van 0-5","Van 6-11","Van 12 or older","SUV 0-5","SUV 6-11","SUV 12 or older",
               "Pick-up 0-5","Pick-up 6-11","Pick-up 12 or older","Motor-bike")
# Car 0 - 5 years old
ivmt2 <- c("uno", "INCOME1", "INCOME4", "NUMCHILD", "WORK3", "Prop_HH_INC5", "RATE_AUTO10");
mcoef2 <- c(-4.542, -0.4, 0.164, -0.189, 0.173, -1.013, -13.833);

# Car 6 - 11 years old
ivmt3 <- c("uno", "WORK2", "INCOME2");
mcoef3 <- c(-5.057, -0.157, 0.132);

# Car 12 years or older
ivmt4 <- c("uno", "INCOME1", "Retired", "Prop_HH_INC1");
mcoef4 <- c(-5.846, 0.574, 0.202, 0.574);

# Van 0 - 5 years old
ivmt5 <- c("uno", "NUMCHILD", "WORK2", "TAZ_den_Q1", "RATE_AUTO30");
mcoef5 <- c(-7.479, 0.379, -0.388, -0.307, -1.717);

# Van 6-11 years old
ivmt6 <- c("uno", "NUMCHILD", "TAZ_den_Q1", "INCOME2");
mcoef6 <- c(-7.579, 0.331, -0.258, 0.269);

# Van 12 years or older
ivmt7 <- c("uno", "HHSIZE", "TAZ_den_Q1", "INCOME1");
mcoef7 <- c(-8.897, 0.138, 0.676, 0.659);

# SUV 0 - 5 years old
ivmt8 <- c("uno", "INCOME1", "WORK2", "Retired", "Prop_HH_INC1", "RATE_AUTO10", "TAZ_den_Q3");
mcoef8 <- c(-5.815, -1.014, 0.169, -0.167, -1.475, -17.83, 0.267);

# SUV 6-11 years old
ivmt9 <- c("uno", "INCOME3", "HSIZE4", "SF_Own", "AUTO30_Q1");
mcoef9 <- c(-7.371, 0.257, 0.333, 0.86, -0.301);

# SUV 12 years or older
ivmt10 <- c("uno", "INCOME4", "child_pres", "SF", "TAZ_den_Q2");
mcoef10 <- c(-7.030, 0.36, 0.313, -0.736, -0.361);

# Pick-up 0-5 years old
ivmt11 <- c("uno", "INCOME5", "HSIZE1", "RUR", "Prop_SFHH");
mcoef11 <- c(-7.616, 0.244, -0.978, 0.239, 0.74);

# Pick-up 6-11 years old
ivmt12 <- c("uno", "RUR", "Retired", "INCOME4", "AUTO10_Q1");
mcoef12 <- c(-6.583, 0.153, -0.353, 0.164, -0.229);

# Pick-up 12 years or older
ivmt13 <- c("uno", "Prop_HH_INC1", "INCOME2", "child_pres", "AUTO10_Q1");
mcoef13 <- c(-7.177, 1.283, 0.345, -0.25, -0.335);

# Motorbike
ivmt14 <- c("uno", "RUR", "SF_Own", "HSIZE1");
mcoef14 <- c(-8.190, 0.705, 0.752, -0.567);


ivmts <- list(ivmt2,ivmt3,ivmt4,ivmt5,ivmt6,ivmt7,ivmt8,ivmt9,ivmt10,ivmt11,ivmt12,ivmt13,ivmt14);
ivmts_coef <- c(mcoef2,mcoef3,mcoef4,mcoef5,mcoef6,mcoef7,mcoef8,mcoef9,mcoef10,mcoef11,mcoef12,mcoef13,mcoef14);



#### Translation parameters
# Car 0 - 5 years old
ivgt2 <- c("uno",  "INCOME1", "INCOME2", "INCOME3");
gcoef2 <- c(8.072, 0.3, -0.35, -0.15);

# Car 6 - 11 years old
ivgt3 <- c("uno", "INCOME1");
gcoef3 <- c(7.732, 0.35);

# Car 12 years or older
ivgt4 <- c("uno",  "INCOME1");
gcoef4 <- c(7.806, 0.2);

# Van 0 - 5 years old
ivgt5 <- c("uno",  "INCOME1", "INCOME2");
gcoef5 <- c(10.889, 1.55, -0.5);

# Van 6-11 years old
ivgt6 <- c("uno",	"INCOME2", "INCOME3");
gcoef6 <- c(9.810, 0.5, 1.5);

# Van 12 years or older
ivgt7 <- c("uno",	"INCOME1", "INCOME2", "INCOME3");
gcoef7 <- c(10.949, -0.2, -1.0, 3.0);

# SUV 0 - 5 years old
ivgt8 <- c("uno",	"INCOME1", "INCOME2");
gcoef8 <- c(9.233, 0.35, -0.65);

# SUV 6-11 years old
ivgt9 <- c("uno");
gcoef9 <- c(9.024);

# SUV 12 years or older
ivgt10 <- c("uno");
gcoef10 <- c(9.156);

# Pick-up 0-5 years old
ivgt11 <- c("uno");
gcoef11 <- c(10.134);

# Pick-up 6-11 years old
ivgt12 <- c("uno");
gcoef12 <- c(9.001);

# Pick-up 12 years or older
ivgt13 <- c("uno");
gcoef13 <- c(9.164);

# Motorbike
ivgt14 <- c("uno", "INCOME1", "INCOME2", "INCOME3");
gcoef14 <- c(7.507, 1.0, -1.0, 0.5);


ivgts <- list(ivgt2,ivgt3,ivgt4,ivgt5,ivgt6,ivgt7,ivgt8,ivgt9,ivgt10,ivgt11,ivgt12,ivgt13,ivgt14);
ivgts_coef <- c(gcoef2,gcoef3,gcoef4,gcoef5,gcoef6,gcoef7,gcoef8,gcoef9,gcoef10,gcoef11,gcoef12,gcoef13,gcoef14);



mdcev <- VehicleFleet_model(data1, ivmts, ivmts_coef, ivgts, ivgts_coef, controls);

#mdcev$ID <- NULL;
data1 <<- cbind(data1,mdcev);

write.table(data1, file = mdcev_output, row.names=FALSE, sep=",");
#rm(mdcev);


# 4 - Heuristic mileage reallocation algorithm

Tolerance <<- 0.035; # Tolerance limit to match predicted bodytype distribution
max_iteration <<- 5; # Maximum number of iterations for the mileage re-allocation algorithm
nc <<- 14;

diff_body <<- c(1.0, 1.0, 1.0, 1.0, 1.0);
iter <<- 1;

print("Running heuristic mileage reallocation algorithm....");
source(paste(c(current_folder,'4_Heuristic_mileage_reallocation.r'),collapse=''));


# 5 - MNL model for number of vehicle body types
# Calibrated coefficients

print("Checking tolerance limits .......");
source(paste(c(current_folder,'5_Number_of_bodytypes.r'),collapse=''));

coefs_name_0type <- c('const', 'INCOME1', 'INCOME2', 'OWN', 'HSIZE1', 'WORK0', 'POPDEN');
coefs_0type <- c(0.68, 2.093, 0.608, -1.430, 2.109, 1.145, 0.00011);
coefs_name_1type <- c('const', 'INCOME1', 'INCOME2', 'HSIZE1', 'Prop_MFHH', 'child_pres');
coefs_1type <- c(3.672, 1.015, 0.646, 1.995, 0.260, -0.333);
coefs_name_2type <- c('const', 'RUR', 'INCOME4', 'INCOME5', 'HSIZE4','OWN');
coefs_2type <- c(2.956, 0.190, 0.353, 0.230, 0.303, 0.983);
coefs_name_3type <- c('const', 'OWN', 'NUMADLT', 'WORK3', 'POPDEN', 'child_pres');
coefs_3type <- c(0.0, 1.418, 0.548, 0.460, -0.0001, 0.364);
coefs_name_4type <- c('const');
coefs_4type <- c(0.0);

coef_names_type <- list(coefs_name_0type, coefs_name_1type, coefs_name_2type , coefs_name_3type, coefs_name_4type);
coef_values_type <- list(coefs_0type, coefs_1type, coefs_2type, coefs_3type, coefs_4type);


infor <- NULL;
tt3 <-NULL;
while ((iter <= max_iteration) & (diff_body[1] > Tolerance | diff_body[2] > Tolerance | diff_body[3] > Tolerance | diff_body[4] > Tolerance | diff_body[5] > Tolerance)) {
  
  print(paste("Iteration", iter));
  
  
  tt2 <- Heuristic_mileage(data1, mdcev);
  indfor <- tt2[[1]];
  bothnum <- tt2[[2]];
  bothmile <- tt2[[3]];
  tt3 <- NumBodytypes(data1, indfor, bothnum, bothmile, coef_names_type, coef_values_type);
  names(tt3) <- c("NumBodyType_HMR","NumBodyType_MNL");

  print(diff_body);
  iter <- iter+1;
}

if(iter > max_iteration){
  print("Consider adjusting the tolerance criteria and/or re-calibrate MDCEV model");
}

Sys.sleep(3);

indfor$ID <- NULL;
names(indfor) <- alt_names <- c("NONMOTOR_HU","CAR1_HU","CAR2_HU","CAR3_HU","VAN1_HU","VAN2_HU","VAN3_HU","SUV1_HU","SUV2_HU","SUV3_HU","PICK1_HU",
                               "PICK2_HU","PICK3_HU","MOTER_HU");
data1 <<- cbind(data1,indfor);
data1 <<- cbind(data1, tt3);
write.table(data1, file = mdcev_output, row.names=FALSE, sep=",");
rm(tt3);
names(indfor) <- alt_names <- c("NONMOTOR","CAR1","CAR2","CAR3","VAN1","VAN2","VAN3","SUV1","SUV2","SUV3","PICKUP1",
                                "PICKUP2","PICKUP3","MOTER");



# Ordered Probit Model for Count of Car, VAN, SUV, and Pick-up Truck
#
# 6.1 - Ordered probit Model for car count (Calibrated coefficients)

print("Running count models for CAR .......");
source(paste(c(current_folder,'6.1_Car_count.r'),collapse=''));

coef_names_car <- c('const', 'YCAR0_5', 'YCAR6_11', 'INCOME2', 'INCOME5', 'WORK3', 'NUMADLT', 'HSIZE1', 'HSIZE4', 'OWN', 'MU (Threshold)');
coefs_car <- c(-4.494, 0.955, 1.054, -0.105, 0.193, 0.409, 0.473, -0.417, -0.258, 0.285, 1.450);

tt1 <- car_count_model(data1, indfor, coef_names_car, coefs_car);
data1 <- cbind(data1,tt1);
rm(tt1);


# 6.2 - Ordered probit model for van count (Calibrated coefficients)

print("Running count models for VAN .......");
source(paste(c(current_folder,'6.2_Van_count.r'),collapse=''));

coef_names_van <- c('const', 'YVAN6_11', 'YVAN12', 'INCOME2', 'vanmile', 'TAZ_den_Q3', 'AUTO10_Q1', 'EMPDEN');
coefs_van <- c(-4.620, 1.952, 2.572, -0.593, 0.00006, 0.597, 0.867, -0.00023);

tt1 <- van_count_model(data1, indfor, coef_names_van, coefs_van);
data1 <- cbind(data1,tt1);
rm(tt1);  


# 6.3 - Ordered probit model for SUV count (Calibrated coefficients)

print("Running count models for SUV .......");
source(paste(c(current_folder,'6.3_SUV_count.r'),collapse=''));

coef_names_suv <- c('const', 'YSUV0_5', 'YSUV12', 'NUMADLT', 'INCOME2', 'INCOME5', 'suvmile', 'WORK3', 'RATE_TRANSIT30');
coefs_suv <- c(-3.640, 0.905, 1.496, 0.455, -0.535, 0.398, 0.00004, -0.459, -34.163);

tt1 <- SUV_count_model(data1, indfor, coef_names_suv, coefs_suv);
data1 <- cbind(data1,tt1);
rm(tt1);


# 6.4 - Ordered probit model for pick-up count (Calibrated coefficients)

print("Running count models for PICKUP .......");
source(paste(c(current_folder,'6.4_Pickup_count.r'),collapse=''));

coef_names_pick <- c('const', 'YPICK0_5', 'YPICK6_11', 'pickmile', 'INCOME5', 'WORK3', 'OWN', 'empden_per_100', 'TAZ_den_Q3', 'AUTO30_Q1');
coefs_pick <- c(-3.946, 0.869, 1.016, 0.000058, -0.287, 0.460, 0.808, 0.0078, 0.299, 0.268);

tt1 <- pickup_count_model(data1, indfor, coef_names_pick, coefs_pick);
data1 <- cbind(data1,tt1);
rm(tt1);


### Final result of Vehicle Fleet Composition model 
write.table(data1, file = mdcev_output, row.names=FALSE, sep=",");

print("Successfully done!.")